import os
import unittest
from pyaudioai.OpenAiResponse import OpenAiResponse

__author__ = "Manuel Eberle (info@manueleberle.de)"

class TestOpenAiResponse(unittest.TestCase):

    openai_obj = OpenAiResponse(os.getenv("OPENAI_API_KEY"))

    def test_init(self):
        assert self.openai_obj.model == "gpt-3.5-turbo"
        assert self.openai_obj.audio_model == "whisper-1"

    def test_transcribe(self):
        print(os.getcwd())
        assert "rechne" in self.openai_obj.transcribe("tests/audio/1plus3.m4a").lower()
    
    def test_transcribe_error(self):
        with self.assertRaises(FileNotFoundError):
            self.openai_obj.transcribe("notavailable.m4a")

    def test_response(self):
        assert "4" in self.openai_obj.get_text_response("tests/audio/1plus3.m4a")


if __name__ == '__main__':
    unittest.main()



import os
import openai
from gtts import gTTS
from tempfile import NamedTemporaryFile
from playsound import playsound

__author__ = "Manuel Eberle (info@manueleberle.de)"

class OpenAiResponse:
    """
    A class used to get a text or an audi response from OpenAI ChatGPT.

    Attributes
    ----------
    api_key : str
        key to authenticate through OpenAI API https://platform.openai.com/account/api-keys
    model : str
        select a ChatGPT model https://platform.openai.com/docs/guides/gpt (default gpt-3.5-turbo)
    audio_model : str
        select a speech to text model https://platform.openai.com/docs/guides/speech-to-text (default whisper-1)
    """

    def __init__(self, api_key, model = "gpt-3.5-turbo", audio_model = "whisper-1"):
        """
        Parameters
        ----------
        api_key : str
            key to authenticate through OpenAI API https://platform.openai.com/account/api-keys
        model : str
            select a ChatGPT model https://platform.openai.com/docs/guides/gpt (default gpt-3.5-turbo)
        audio_model : str
            select a speech to text model https://platform.openai.com/docs/guides/speech-to-text (default whisper-1)
        """

        openai.api_key = api_key
        self.model = model
        self.audio_model = audio_model

    def transcribe(self, audio_file):
        """Transcribe an audio file to text.

        Parameters
        ----------
        audio_file : str, required
            Path to an audio file which contains a question

        Raises
        ------
        FileNotFoundError
            If the audio file is not found
        """

        if not os.path.exists(audio_file):
            raise FileNotFoundError("[ERROR] file '{}' not found".format(audio_file))
        
        audio_stream = open(audio_file, "rb")
        transcript = openai.Audio.transcribe(self.audio_model, audio_stream)
        return transcript.text

    def get_text_response(self, audio_file):
        """Get a text response of an audio file which contains a question.

        Parameters
        ----------
        audio_file : str, required
            Path to an audio file which contains a question
        
        Returns
        -------
        str
            response from ChatGPT as text
        """

        response = openai.ChatCompletion.create(
        model = self.model,
        messages = [
                {"role": "system", "content": self.transcribe(audio_file)}
            ]
        )
        return response.choices[0].message.content
    
    def get_audio_response(self, audio_file):
        """Get an audio response of an audio file which contains a question.
        The function converts the text repsonse with TTS to an audio object
        and plays the audio.

        Parameters
        ----------
        audio_file : str, required
            Path to an audio file which contains a questio
        """
        text_response = self.get_text_response(audio_file)
        print("[INFO] {}".format(text_response))
        
        gTTS(text=text_response, lang='de').write_to_fp(voice := NamedTemporaryFile())
        playsound(voice.name)
        voice.close()


import os
import wave
import pyaudio

__author__ = "Manuel Eberle (info@manueleberle.de)"

class RecordQuestion:
    """
    A class used to record an audio file with the computers microphone.

    Attributes
    ----------
    recording_time : int
        recording time in seconds
    audio_file : str
        Path to an audio file where the question is stored temporarily.
    """

    __format = pyaudio.paInt16
    __channel = 2
    __rate = 44100
    __chunk = 1024

    def __init__(self, recording_time, audio_file):
        """
        Parameters
        ----------
        recording_time : int
            recording time in seconds
        audio_file : str
            Path to an audio file where the question is stored temporarily.
        """

        self.recording_time = recording_time
        self.audio_file = audio_file

    def record(self):
        """Record the audio file.
        """

        pyaudio_obj = pyaudio.PyAudio()
        audio_stream = pyaudio_obj.open(format=self.__format, channels=self.__channel, rate=self.__rate, input=True, frames_per_buffer=self.__chunk)
        
        print("[INFO] Start recording!")
        audio_frames = []
        for _ in range(0, int(self.__rate / self.__chunk * self.recording_time)):
            data_stream = audio_stream.read(self.__chunk)
            audio_frames.append(data_stream)
        
        audio_stream.stop_stream()
        audio_stream.close()
        pyaudio_obj.terminate()
        print("[INFO] Stop recording!")
        
        audio_output = wave.open(self.audio_file, 'wb')
        audio_output.setnchannels(self.__channel)
        audio_output.setsampwidth(pyaudio_obj.get_sample_size(self.__format))
        audio_output.setframerate(self.__rate)
        audio_output.writeframes(b''.join(audio_frames))
        audio_output.close()

    def delete_file(self):
        """Delete the temporary generated audio file.
        """

        os.remove(self.audio_file)


import os
import unittest
from pyaudioai.RecordQuestion import RecordQuestion

__author__ = "Manuel Eberle (info@manueleberle.de)"

class TestRecordQuestion(unittest.TestCase):

    def test_init(self):
        record_obj = RecordQuestion(5, "question.m4a")
        assert record_obj.recording_time == 5
        assert record_obj.audio_file == "question.m4a"

    def test_delete_file(self):
        mock_file_name = "test.txt"
        record_obj = RecordQuestion(5, mock_file_name)

        mock_file = open(mock_file_name, "w")
        mock_file.close()

        record_obj.delete_file()
        self.assertFalse(os.path.exists(mock_file_name))


if __name__ == '__main__':
    unittest.main()



import setuptools

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name='pyaudioai',
    author='Manuel Eberle',
    author_email='info@manueleberle.de',
    description='Get an audio response from ChatGPT',
    keywords='chatgpt, openai, audio, tts, pypi, package',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/eberlemanuel/pyaudioai',
    project_urls={
        'Documentation': 'https://github.com/eberlemanuel/pyaudioai',
        'Bug Reports': 'https://github.com/eberlemanuel/pyaudioai/issues',
        'Source Code': 'https://github.com/eberlemanuel/pyaudioai',
        'Source Code Documenetation': 'https://eberlemanuel.github.io/pyaudioai',
    },
    package_dir={'': 'src'},
    packages=setuptools.find_packages(where='src'),
    classifiers=[
        'Development Status :: 5 - Production/Stable',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3 :: Only',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.8',
    install_requires=['openai', 'gTTS', 'playsound', 'pyaudio', 'wave'],
    extras_require={
        'dev': ['check-manifest'],
        'test': ['coverage'],
    },
)
